"use strict";
/* This is a conversion from BLAS to Typescript/Javascript
Copyright (C) 2018  Jacob K.F. Bogers  info@mail.jacob-bogers.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.ctpsv = void 0;
const f_func_1 = require("../../../f_func");
const norm_lower_1 = require("./norm-lower");
const norm_upper_1 = require("./norm-upper");
const trans_lower_1 = require("./trans-lower");
const trans_upper_1 = require("./trans-upper");
/*
 *>
 *> CTPSV  solves one of the systems of equations
 *>
 *>    A*x = b,   or   A**T*x = b,   or   A**H*x = b,
 *>
 *> where b and x are n element vectors and A is an n by n unit, or
 *> non-unit, upper or lower triangular matrix, supplied in packed form.
 *>
 *> No test for singularity or near-singularity is included in this
 *> routine. Such tests must be performed before calling this routine.
 */
function ctpsv(uplo, trans, diag, n, ap, x, incx) {
    if (x.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('x.i'));
    }
    if (ap.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('ap.i'));
    }
    // faster then String.toLowerCase()
    const ul = (0, f_func_1.lowerChar)(uplo);
    const tr = (0, f_func_1.lowerChar)(trans);
    const dg = (0, f_func_1.lowerChar)(diag);
    let info = 0;
    if (!'ul'.includes(ul)) {
        info = 1;
    }
    else if (!'ntc'.includes(tr)) {
        info = 2;
    }
    else if (!'un'.includes(dg)) {
        info = 3;
    }
    else if (n < 0) {
        info = 4;
    }
    else if (incx === 0) {
        info = 7;
    }
    if (info !== 0) {
        throw new Error((0, f_func_1.errWrongArg)('ctpsv', info));
    }
    //*     Quick return if possible.
    if (n === 0)
        return;
    const noconj = tr === 't';
    const nounit = dg === 'n';
    const kx = incx < 0 ? 1 - (n - 1) * incx : 1;
    let proc;
    if (tr === 'n') {
        if (ul === 'u') {
            proc = norm_upper_1.normUpper;
            //console.log(`normHigher`);
        }
        else {
            //console.log(`normLower`);
            proc = norm_lower_1.normLower;
        }
    }
    else {
        if (ul === 'u') {
            //console.log(`transUpper`);
            proc = trans_upper_1.transUpper;
        }
        else {
            //console.log(`transLower`);
            proc = trans_lower_1.transLower;
        }
    }
    return proc(kx, noconj, nounit, x, incx, ap, n);
}
exports.ctpsv = ctpsv;
//# sourceMappingURL=index.js.map