"use strict";
/* This is a conversion from BLAS to Typescript/Javascript
Copyright (C) 2018  Jacob K.F. Bogers  info@mail.jacob-bogers.com

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.cgemm = void 0;
const f_func_1 = require("../../../f_func");
const AB_1 = require("./AB");
const AconjB_1 = require("./AconjB");
const AtransB_1 = require("./AtransB");
const conjAB_1 = require("./conjAB");
const conjAconjB_1 = require("./conjAconjB");
const conjAtransB_1 = require("./conjAtransB");
const transAB_1 = require("./transAB");
const transAconjB_1 = require("./transAconjB");
const transAtransB_1 = require("./transAtransB");
const { max } = Math;
function cgemm(transA, transB, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc) {
    const trA = (0, f_func_1.lowerChar)(transA);
    const trB = (0, f_func_1.lowerChar)(transB);
    if (a.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('a.i'));
    }
    if (b.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('b.i'));
    }
    if (c.i === undefined) {
        throw new Error((0, f_func_1.errMissingIm)('c.i'));
    }
    //const notA = trA === 'n';
    //const notB = trB === 'n';
    const nrowA = trA === 'n' ? m : k;
    const nrowB = trB === 'n' ? k : m;
    //* Test the input parameters.
    let info = 0;
    if (!'ntc'.includes(trA)) {
        info = 1;
    }
    else if (!'ntc'.includes(trB)) {
        info = 2;
    }
    else if (m < 0) {
        info = 3;
    }
    else if (n < 0) {
        info = 4;
    }
    else if (k < 0) {
        info = 5;
    }
    else if (lda < max(1, nrowA)) {
        info = 8;
    }
    else if (ldb < max(1, nrowB)) {
        info = 10;
    }
    else if (ldc < max(1, m)) {
        info = 13;
    }
    if (info !== 0) {
        throw new Error((0, f_func_1.errWrongArg)('cgemm', info));
    }
    const alphaIsZero = alpha.re === 0 && alpha.im === 0;
    const betaIsOne = beta.re === 1 && beta.im === 0;
    const betaIsZero = beta.re === 0 && beta.im === 0;
    // Quick return if possible.
    if (m === 0 || n === 0 || ((alphaIsZero || k === 0) && betaIsOne)) {
        return;
    }
    // And when  alpha.eq.zero.
    if (alphaIsZero) {
        //console.log('alpha is zero')
        if (betaIsZero) {
            //fast shortcut
            for (let j = 1; j <= n; j++) {
                c.setCol(j, 1, m, 0);
            }
        }
        else {
            for (let j = 1; j <= n; j++) {
                const coorCJ = c.colOfEx(j);
                for (let i = 1; i <= m; i++) {
                    const { re, im } = (0, f_func_1.mul_cxr)(beta, c.r[coorCJ + i], c.i[coorCJ + i]);
                    c.r[coorCJ + i] = re;
                    c.i[coorCJ + i] = im;
                }
            }
        }
        return;
    }
    let proc;
    //    Start the operations.
    if (trA === 'n' && trB === 'n') {
        //Form  C := alpha*A*B + beta*C.
        //console.log('AB');
        proc = AB_1.AB;
    }
    else if (trA === 'n' && trB === 'c') {
        //Form  C := alpha*A*B**H + beta*C.
        proc = AconjB_1.AconjB;
    }
    else if (trA === 'n' && trB === 't') {
        //Form  C := alpha*A*B**T + beta*C
        proc = AtransB_1.AtransB;
    }
    else if (trA === 'c' && trB === 'n') {
        // Form  C := alpha*A**H*B + beta*C.
        proc = conjAB_1.conjAB;
    }
    else if (trA === 'c' && trB === 'c') {
        //  Form  C := alpha*A**H*B**H + beta*C.
        proc = conjAconjB_1.conjAconjB;
    }
    else if (trA === 'c' && trB === 't') {
        //  Form  C := alpha*A**H*B**T + beta*C
        proc = conjAtransB_1.conjAtransB;
    }
    else if (trA === 't' && trB === 'n') {
        //Form  C := alpha*A**T*B + beta*C
        proc = transAB_1.transAB;
    }
    else if (trA === 't' && trB === 'c') {
        //Form  C := alpha*A**T*B**H + beta*C
        proc = transAconjB_1.transAconjB;
    } /*if (trA === 't' && trB === 't')*/
    else {
        //  //  Form  C := alpha*A**T*B**T + beta*C
        proc = transAtransB_1.transAtransB;
    }
    // throw new Error('unreachable code');
    //console.log('name', proc.name);
    return proc(betaIsZero, betaIsOne, beta, alpha, a, b, c, n, m, k);
}
exports.cgemm = cgemm;
//# sourceMappingURL=index.js.map